==> hlog-0.16-rc1.tar.gz: hlog-0.16-rc1/xlua/adif.c <==
/*
 * Copyright (c) 2020 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "xlua_impl.h"
#include <hlog/adif.h>

/*
 * Expose adif writing API
 */

/* return a string with an ADIF header [-0,+1,e] */
static int xlua_adif_write_header(lua_State *state)
{
	struct buffer buf;
	char tmp[4096];
	int ret;

	xlua_checkstack(state, 0, 1);

	buffer_init_static(&buf, tmp, 0, sizeof(tmp), true);

	ret = adif_write_header(&buf);
	if (ret)
		return luaL_error(state, "failed to serialize string: %s",
				  xstrerror(ret));

	lua_pushlstring(state, buffer_data(&buf), buffer_size(&buf));

	return 1;
}

/* return a string with an ADIF QSO [-1,+1,e] */
static int xlua_adif_write_qso(lua_State *state)
{
	struct buffer buf;
	struct qso *qso;
	char tmp[4096];
	int ret;

	xlua_checkstack(state, 1, 1);

	qso = xlua_toqso(state, -1);
	ASSERT3P(qso, !=, NULL);

	buffer_init_static(&buf, tmp, 0, sizeof(tmp), true);

	ret = adif_write_qso(&buf, qso);
	if (ret)
		return luaL_error(state, "failed to serialize qso: %s",
				  xstrerror(ret));

	lua_pushlstring(state, buffer_data(&buf), buffer_size(&buf));

	return 1;
}

static const luaL_Reg adif_funcs[] = {
	{ "write_header", xlua_adif_write_header },
	{ "write_qso", xlua_adif_write_qso },
	{ NULL, NULL },
};

/* register the adif library [-0,+0,e] */
void xlua_pushlib_adif(lua_State *state)
{
	luaL_newlib(state, adif_funcs);
}

==> hlog-0.17.tar.gz: hlog-0.17/xlua/adif.c <==
/*
 * Copyright (c) 2020-2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "xlua_impl.h"
#include <hlog/adif.h>

/*
 * Expose adif writing API
 */

/*
 * lookup ADIF <mode,submode> and return the hlog full mode or nil if not
 * found [-2,+1,e]
 */
static int xlua_adif_lookup_adif_mode(lua_State *L)
{
	const char *mode, *submode;

	xlua_checkstack(L, 2, 2, 1);

	mode = luaL_checkstring(L, -2);

	if (lua_isnil(L, -1))
		submode = NULL;
	else
		submode = luaL_checkstring(L, -1);

	if (!mode)
		return luaL_error(L, "mode cannot be nil");

	mode = adif_mode_lookup_by_adif_mode(mode, submode);
	if (IS_ERR(mode))
		lua_pushnil(L);
	else
		lua_pushstring(L, mode);

	return 1;
}

/* return a string with an ADIF header [-0,+1,e] */
static int xlua_adif_write_header(lua_State *L)
{
	struct buffer buf;
	char tmp[4096];
	int ret;

	xlua_checkstack(L, 0, 0, 1);

	buffer_init_static(&buf, tmp, 0, sizeof(tmp), true);

	ret = adif_append_header(&buf);
	if (ret)
		return luaL_error(L, "failed to serialize string: %s",
				  xstrerror(ret));

	lua_pushlstring(L, buffer_data(&buf), buffer_size(&buf));

	return 1;
}

/* return a string with an ADIF QSO [-1,+1,e] */
static int xlua_adif_write_qso(lua_State *L)
{
	struct buffer buf;
	struct qso *qso;
	char tmp[4096];
	int ret;

	xlua_checkstack(L, 1, 1, 1);

	qso = xlua_toqso(L, -1);
	ASSERT3P(qso, !=, NULL);

	buffer_init_static(&buf, tmp, 0, sizeof(tmp), true);

	ret = adif_append_qso(&buf, qso, false);
	if (ret)
		return luaL_error(L, "failed to serialize qso: %s",
				  xstrerror(ret));

	lua_pushlstring(L, buffer_data(&buf), buffer_size(&buf));

	return 1;
}

static const luaL_Reg adif_funcs[] = {
	{ "lookup_adif_mode", xlua_adif_lookup_adif_mode },
	{ "write_header", xlua_adif_write_header },
	{ "write_qso", xlua_adif_write_qso },
	{ NULL, NULL },
};

/* register the adif library [-0,+0,e] */
void xlua_pushlib_adif(lua_State *L)
{
	luaL_newlib(L, adif_funcs);
}
